@@ -5,7 +5,7 @@
 (autoload 'custom-add-loads "cus-load")
 
 (custom-add-loads 'tools '("ediff"))
-(custom-add-loads 'ediff-hook '("ediff-init"))
+(custom-add-loads 'ediff-hook '("ediff-init" "ediff-mult"))
 (custom-add-loads 'ediff-diff '("ediff-diff"))
 (custom-add-loads 'ediff-window '("ediff-wind"))
 (custom-add-loads 'frames '("ediff-wind"))
