@@ -397,32 +397,69 @@ are two possible targets for this patch.  However, these files do not exist."
 	 (set-window-buffer ediff-window-B ediff-patch-diagnostics))
 	(t (display-buffer ediff-patch-diagnostics 'not-this-window))))
 
-(defun ediff-get-patch-buffer ()
-  "Obtain patch buffer.  If patch is already in a buffer---use it.
-Else, read patch file into a new buffer."
+;; prompt for file, get the buffer
+(defun ediff-prompt-for-patch-file ()
   (let ((dir (cond (ediff-patch-default-directory) ; try patch default dir
 		   (ediff-use-last-dir ediff-last-dir-patch)
-		   (t default-directory)))
-	patch-buf)
-    (if (let ((last-nonmenu-event t) ; Emacs: don't use dialog box
-	      last-command-event)    ; XEmacs: don't use dialog box
-	  (y-or-n-p "Is the patch already in a buffer? "))
-	(setq patch-buf
-	      (get-buffer
-	       (read-buffer
-		"Which buffer contains the patch? "
-		(ediff-other-buffer
-		 (if (eq (next-window (selected-window)) (selected-window))
-		     ;; only one window in frame --- don't skip current buff
-		     ""
-		   ;; >1 window --- skip current buff, assuming this is the one
-		   ;; to patch, not the one that has the patch
-		   (current-buffer)))
-		'must-match)))
-      (setq patch-buf
-	    (find-file-noselect
-	     (read-file-name "Which file contains the patch? "
-			     dir nil 'must-match))))
+		   (t default-directory))))
+    (find-file-noselect
+     (read-file-name
+      (format "Patch is in file:%s "
+	      (cond ((and buffer-file-name
+			  (equal (expand-file-name dir)
+				 (file-name-directory buffer-file-name)))
+		     (concat
+		      " (default "
+		      (file-name-nondirectory buffer-file-name)
+		      ")"))
+		    (t "")))
+      dir buffer-file-name 'must-match))
+    ))
+
+
+;; Try current buffer, then the other window's buffer. Else, give up.
+(defun ediff-prompt-for-patch-buffer ()
+  (get-buffer
+   (read-buffer
+    "Patch is in buffer: "
+    (cond ((save-excursion
+	     (goto-char (point-min))
+	     (re-search-forward ediff-context-diff-label-regexp nil t))
+	   (current-buffer))
+	  ((save-window-excursion
+	     (other-window 1)
+	     (save-excursion
+	       (goto-char (point-min))
+	       (and (re-search-forward ediff-context-diff-label-regexp nil t)
+		    (current-buffer)))))
+	  ((save-window-excursion
+	     (other-window -1)
+	     (save-excursion
+	       (goto-char (point-min))
+	       (and (re-search-forward ediff-context-diff-label-regexp nil t)
+		    (current-buffer)))))
+	  (t nil))
+    'must-match)))
+
+
+(defun ediff-get-patch-buffer (&optional arg patch-buf)
+  "Obtain patch buffer.  If patch is already in a buffer---use it.
+Else, read patch file into a new buffer. If patch buffer is passed as an
+optional argument, then use it."
+  (let ((last-nonmenu-event t) ; Emacs: don't use dialog box
+	last-command-event)    ; XEmacs: don't use dialog box
+
+    (cond ((ediff-buffer-live-p patch-buf))
+	  ;; even prefix arg: patch in buffer
+	  ((and (integerp arg) (eq 0 (mod arg 2)))
+	   (setq patch-buf (ediff-prompt-for-patch-buffer)))
+	  ;; odd prefix arg: get patch from a file
+	  ((and (integerp arg) (eq 1 (mod arg 2)))
+	   (setq patch-buf (ediff-prompt-for-patch-file)))
+	  (t (setq patch-buf
+		   (if (y-or-n-p "Is the patch already in a buffer? ")
+		       (ediff-prompt-for-patch-buffer)
+		     (ediff-prompt-for-patch-file)))))
     
     (ediff-with-current-buffer patch-buf
       (goto-char (point-min))
@@ -678,12 +715,11 @@ you can still examine the changes via M-x ediff-files"
     (setq startup-hooks
 	  ;; this sets various vars in the meta buffer inside
 	  ;; ediff-prepare-meta-buffer
-	  (cons (` (lambda ()
-		     ;; tell what to do if the user clicks on a session record
-		     (setq ediff-session-action-function
-			   'ediff-patch-file-form-meta
-			   ediff-meta-patchbufer patch-buf)
-		     ))
+	  (cons `(lambda ()
+		   ;; tell what to do if the user clicks on a session record
+		   (setq ediff-session-action-function
+			 'ediff-patch-file-form-meta
+			 ediff-meta-patchbufer patch-buf) )
 		startup-hooks))
     (setq meta-buf (ediff-prepare-meta-buffer 
 		    'ediff-filegroup-action
